@@ -132,7 +132,7 @@ class Interface:
         t = trange(steps, desc='Bar desc', leave=True)
 
         for step in t:
-            loss[step] = svi.step(i = step + 1) / num_observations
+            loss[step] = svi.step(i = step + 1) / (num_observations * num_segments)
             elb = loss[step]
             t.set_description('ELBO: {:.9f}  '.format(elb))
             t.refresh()
@@ -174,6 +174,8 @@ class Interface:
 
         params["CNA"] = torch.argmax(params["CNV_probabilities"] , axis=(2)) + 1
         
+        print("", flush=True)
+
         print("Computing assignment probabilities", flush=True)
         discrete_params = self._model.calculate_cluster_assignements(params)
 
@@ -204,11 +206,15 @@ class Interface:
         n_params = calculate_number_of_params(params)
 
         N = 0
+        entropy_rna = 0
+        lk_rna = 0
         if "data_rna" in self._model._data :
             N += self._model._data['data_rna'].shape[1]
             lk_rna = self._model.likelihood(params, "rna",sum = True)
             entropy_rna = calc_entropy(ca["assignment_probs_rna"])
 
+        lk_atac = 0
+        entropy_atac = 0
         if "data_atac" in self._model._data:
             N +=  self._model._data['data_atac'].shape[1]
             lk_atac = self._model.likelihood(params, "atac",sum = True)
