@@ -6,7 +6,7 @@ from pyro import poutine
 from pyro.infer.autoguide import AutoDelta
 import numpy as np
 from congas.models.Model import Model
-from congas.utils import log_sum_exp, entropy_CNV
+from congas.utils import log_sum_exp, entropy_mixture, entropy_per_segment
 import congas.building_blocks as lks
 
 from sklearn.cluster import KMeans
@@ -19,7 +19,7 @@ class LatentCategorical(Model):
               'theta_shape_rna': None, 'theta_rate_rna': None,'theta_shape_atac': None, 'theta_rate_atac': None,
               'batch_size': None, "init_probs" : 5, 'norm_init_sd_rna' : None, "norm_init_sd_atac" : None,
               'mixture': None, "nb_size_init_atac": None,"nb_size_init_rna": None, "binom_prior_limits" : [10,10000],
-              "likelihood_rna" : "NB", "likelihood_atac" : "NB", 'lambda' : 1, "latent_type" : "D", "Temperature" : 1/100}
+              "likelihood_rna" : "NB", "likelihood_atac" : "NB", 'lambda' : 1, "latent_type" : "D", "Temperature" : 1/100, "equal_sizes_sd" : True}
 
     data_name = set(['data_rna', 'data_atac', 'pld', 'segments', 'norm_factor_rna', 'norm_factor_atac'])
 
@@ -40,7 +40,7 @@ class LatentCategorical(Model):
             weights_rna = pyro.sample('mixture_weights_rna',
                                   dist.Dirichlet((1. / self._params['K']) * torch.ones(self._params['K'])))
         else:
-            N = 1
+            N = 0
 
         if 'data_atac' in self._data:
             I, M = self._data['data_atac'].shape
@@ -49,7 +49,7 @@ class LatentCategorical(Model):
             weights_atac = pyro.sample('mixture_weights_atac',
                                   dist.Dirichlet((1. / self._params['K']) * torch.ones(self._params['K'])))
         else:
-            M = 1
+            M = 0
 
         cat_vector = torch.tensor(np.arange(1, self._params['hidden_dim'] + 1), dtype=torch.float)
 
@@ -58,14 +58,14 @@ class LatentCategorical(Model):
 
             ### RNA segment dependent factors ###
             if 'data_rna' in self._data:
-                if self._params["likelihood_rna"] == "NB":
+                if self._params["likelihood_rna"] == "NB" and self._params["equal_sizes_sd"]:
                     #size_r
                     sizes_rna = pyro.sample("NB_size_rna", dist.Uniform(self._params['binom_prior_limits'][0],
                                                                 self._params['binom_prior_limits'][1]))
 
 
 
-                if self._params["likelihood_rna"] in ["N", "G"]:
+                if self._params["likelihood_rna"] in ["N", "G"] and self._params["equal_sizes_sd"]:
                     # sd_r
                     norm_sd_rna = pyro.sample('norm_sd_rna', dist.Uniform(self._params['a'], self._params['b']))
                     segment_factor_rna = torch.ones(I)
@@ -78,12 +78,12 @@ class LatentCategorical(Model):
             ### ATAC segment dependent factors ###
             if 'data_atac' in self._data:
                 # size_a
-                if self._params["likelihood_atac"] == "NB":
+                if self._params["likelihood_atac"] == "NB" and self._params["equal_sizes_sd"]:
                     sizes_atac = pyro.sample("NB_size_atac", dist.Uniform(self._params['binom_prior_limits'][0],
                                                                 self._params['binom_prior_limits'][1]))
 
 
-                if self._params["likelihood_atac"] in ["N", "G"]:
+                if self._params["likelihood_atac"] in ["N", "G"] and self._params["equal_sizes_sd"]:
                     # sd_a
                     norm_sd_atac = pyro.sample('norm_sd_atac', dist.Uniform(self._params['a'], self._params['b']))
                     segment_factor_atac = torch.ones(I)
@@ -96,21 +96,45 @@ class LatentCategorical(Model):
 
             ### MODALITY INDEPENDENT HIDDEN CNV VALUES ###
             with pyro.plate('components', self._params['K']):
-                # C
+
+                if 'data_atac' in self._data and self._params["likelihood_atac"] == "NB" and not self._params["equal_sizes_sd"]:
+                    sizes_atac = pyro.sample("NB_size_atac", dist.Uniform(self._params['binom_prior_limits'][0],
+                                                                          self._params['binom_prior_limits'][1]))
+                if 'data_atac' in self._data and self._params["likelihood_atac"] in ["N", "G"] and not self._params["equal_sizes_sd"]:
+                    norm_sd_atac = pyro.sample('norm_sd_atac', dist.Uniform(self._params['a'], self._params['b']))
+
+
+                if 'data_rna' in self._data and self._params["likelihood_rna"] == "NB" and not self._params["equal_sizes_sd"]:
+                    sizes_rna = pyro.sample("NB_size_rna", dist.Uniform(self._params['binom_prior_limits'][0],
+                                                                          self._params['binom_prior_limits'][1]))
+                if 'data_rna' in self._data and self._params["likelihood_rna"] in ["N", "G"] and not self._params["equal_sizes_sd"]:
+                    norm_sd_rna = pyro.sample('norm_sd_rna', dist.Uniform(self._params['a'], self._params['b']))
+
+
+                #C
                 cc = pyro.sample("CNV_probabilities", dist.Dirichlet(self._params['probs']))
 
-        ### MEAN FILED-LIKE MODEL ###
-        # E[lk(x_ni | z_nk)] = segment_factor_i * CNA_k * norm_factor_n
-        # CNA_k = rho_k * [1,...,5]
+
+
+                if self._params["latent_type"] == "G":
+                    gumble = tdist.Gumbel(0, 1).sample(cc.shape)
+                    cc_argmax = ((torch.log(cc) + gumble) / Temperature).softmax(-1)
+                else:
+                    cc_argmax = pyro.sample("cc_argmax", dist.OneHotCategorical(cc),infer=dict(baseline={'use_decaying_avg_baseline': True,
+                                         'baseline_beta': 0.95}))
 
 
         lk_rna = 0
+        entropy_per_segments_rna = 0
+        penalty_per_segments_rna = 0
+
         if 'data_rna' in self._data:
 
             with pyro.plate('data_rna', N, batch1):
                 # p(x|z_i) = Poisson(marg(cc * theta * segment_factor))
 
-                if self._params["latent_type"] == "D":
+
+                if self._params["latent_type"] == "M":
 
                     if self._params["likelihood_rna"] == "NB":
                         lk_rna = lks.NB_likelihood2(self, segment_factor_rna, cc, cat_vector, weights_rna, sizes_rna, "rna")
@@ -125,8 +149,7 @@ class LatentCategorical(Model):
                     segment_fact_cat = torch.matmul(segment_factor_rna.reshape([I, 1]),
                                                     cat_vector.reshape([1, self._params['hidden_dim']]))
 
-                    gumble = tdist.Gumbel(0, 1).sample(cc.shape)
-                    cc_argmax = ((torch.log(cc) + gumble)/Temperature).softmax(-1)
+
 
 
                     segment_fact_marg_rna = segment_fact_cat * cc_argmax
@@ -134,23 +157,40 @@ class LatentCategorical(Model):
                     segment_fact_marg_rna = torch.sum(segment_fact_marg_rna, dim=-1)
 
                     if self._params["likelihood_rna"] == "NB":
-                        lk_rna = lks.NB_likelihood(self, segment_fact_marg_rna, weights_rna, sizes_rna, "rna")
+                        lk_rna_aux = lks.NB_likelihood_aux(self, segment_fact_marg_rna, sizes_rna, "rna")
 
                     elif self._params["likelihood_rna"] == "P":
-                        lk_rna = lks.poisson_likelihood(self, segment_fact_marg_rna, weights_rna, "rna")
+                        lk_rna_aux = lks.poisson_likelihood_aux(self, segment_fact_marg_rna, "rna")
 
                     else:
-                        lk_rna = lks.gaussian_likelihood(self, segment_fact_marg_rna, weights_rna, norm_sd_rna, "rna")
+                        lk_rna_aux = lks.gaussian_likelihood_aux(self, segment_fact_marg_rna, norm_sd_rna, "rna")
+
+                    lk_rna_aux += torch.log(weights_rna).reshape([self._params['K'], 1, 1])
+                    norm_lk_rna = log_sum_exp(lk_rna_aux)
+                    #per_segment_ass_rna = torch.exp(lk_rna_aux - norm_lk_rna)
+                    lk_rna = norm_lk_rna.sum()
+
+                    #entropy_per_segments_rna = entropy_per_segment(per_segment_ass_rna)
+
+                    ### penalty per segment ###
+
+                    max = torch.amax(segment_fact_marg_rna, dim = 0)
+
+
+                    penalty_per_segments_rna = torch.linalg.norm((segment_fact_marg_rna) / max, ord=2, dim=0).sum() * N
+
 
         lk_atac = 0
+        entropy_per_segments_atac = 0
+
+        penalty_per_segments_atac = 0
 
         if 'data_atac' in self._data:
 
             with pyro.plate('data_atac', M, batch2):
                 # p(x|z_i) = Poisson(marg(cc * theta * segment_factor))
 
-
-                if self._params["latent_type"] == "D":
+                if self._params["latent_type"] == "M":
 
                     if self._params["likelihood_atac"] == "NB":
                         lk_atac = lks.NB_likelihood2(self, segment_factor_atac,cc, cat_vector, weights_atac, sizes_atac, "atac")
@@ -164,25 +204,42 @@ class LatentCategorical(Model):
                     segment_fact_cat_atac = torch.matmul(segment_factor_atac.reshape([I, 1]),
                                                     cat_vector.reshape([1, self._params['hidden_dim']]))
 
-                    gumble = tdist.Gumbel(0, 1).sample(cc.shape)
-                    cc_argmax = ((torch.log(cc) + gumble) / Temperature).softmax(-1)
+
                     segment_fact_marg_atac = segment_fact_cat_atac * cc_argmax
+
                     segment_fact_marg_atac = torch.sum(segment_fact_marg_atac, dim=-1)
 
 
 
                     if self._params["likelihood_atac"] == "NB":
-                        lk_atac = lks.NB_likelihood(self, segment_fact_marg_atac, weights_atac, sizes_atac, "atac")
+                        lk_atac_aux = lks.NB_likelihood_aux(self, segment_fact_marg_atac, sizes_atac, "atac")
                     elif self._params["likelihood_atac"] == "P":
-                        lk_atac = lks.poisson_likelihood(self, segment_fact_marg_atac, weights_atac, "atac")
-                    else:
-                        lk_atac = lks.gaussian_likelihood(self, segment_fact_marg_atac, weights_atac, norm_sd_atac, "atac")
+                        lk_atac_aux = lks.poisson_likelihood_aux(self, segment_fact_marg_atac, "atac")
 
-        if self._params["latent_type"] == "D":
-            pyro.factor("lk", self._params['lambda'] * lk_rna + (1-self._params['lambda']) * lk_atac -
-                        torch.tensor((self._params['lambda'] * N) * ((1 - self._params['lambda']) * M) / self._params['hidden_dim'], dtype = torch.float) * entropy_CNV(cc))
+                    else:
+                        lk_atac_aux = lks.poisson_likelihood_aux(self, segment_fact_marg_atac, "atac")
+
+                    lk_atac_aux += torch.log(weights_atac).reshape([ self._params['K'], 1, 1])
+                    norm_lk_atac = log_sum_exp(lk_atac_aux)
+                    #per_segment_ass_atac = torch.exp(lk_atac_aux - norm_lk_atac)
+                    lk_atac = norm_lk_atac.sum()
+
+                    #entropy_per_segments_atac = entropy_per_segment(per_segment_ass_atac)
+
+                    ### penalty per segment ###
+                    max = torch.amax(segment_fact_marg_atac, dim = 0)
+                    penalty_per_segments_atac = torch.linalg.norm((segment_fact_marg_atac / max) , ord = 2, dim = 0).sum() * M
+
+
+        if self._params["latent_type"] == "M":
+            pyro.factor("lk", self._params['lambda'] * lk_rna + (1-self._params['lambda']) * lk_atac +
+                        (self._params['lambda'] * N * entropy_mixture(weights_rna)) + ((1 - self._params['lambda']) * M * entropy_mixture(weights_atac)))
         else:
-            pyro.factor("lk", self._params['lambda'] * lk_rna + (1-self._params['lambda']) * lk_atac)
+            #entropy_per_segments = self._params['lambda'] * entropy_per_segments_rna + (1-self._params['lambda']) * entropy_per_segments_atac
+            CN_diff_penalty = self._params['lambda'] * penalty_per_segments_rna + (
+                        1 - self._params['lambda']) * penalty_per_segments_atac
+            lk_total = self._params['lambda'] * lk_rna + (1-self._params['lambda']) * lk_atac
+            pyro.factor("lk", lk_total + CN_diff_penalty)
 
 
 
@@ -212,6 +269,16 @@ class LatentCategorical(Model):
                 return dist.Gamma(self._params['theta_shape_rna'], self._params['theta_rate_rna']).mean
             if site["name"] == "segment_factor_atac":
                 return dist.Gamma(self._params['theta_shape_atac'], self._params['theta_rate_atac']).mean
+            if site["name"] == "cc_argmax":
+                dir_init = self.create_dirichlet_init_values()
+                cc_argmax = torch.zeros_like(dir_init)
+                cc_ones = torch.argmax(dir_init, dim = -1)
+
+                for i in range(cc_argmax.shape[0]):
+                    for j in range(cc_argmax.shape[1]):
+                        cc_argmax[i, j, cc_ones[i, j]] = 1
+
+                return cc_argmax
             raise ValueError(site["name"])
 
         return init_function
@@ -291,24 +358,29 @@ class LatentCategorical(Model):
 
 
 
+
+
         segment_fact_marg = segment_fact * cc_argmax
 
 
+
         segment_fact_marg = torch.sum(segment_fact_marg, dim=-1)
 
 
 
         if self._params["likelihood_{}".format(mod)] == "NB":
-            if self._params["latent_type"] == "D":
-                lk = lks.NB_likelihood_aux(self, segment_fact_marg, inf_params["NB_size_{}".format(mod)], mod)
-            else:
+            if self._params["latent_type"] == "M":
                 lk = lks.NB_likelihood_aux2(self, inf_params["segment_factor_{}".format(mod)],
-                                           inf_params["CNV_probabilities"],
-                                           cat_vector,
-                                           inf_params["NB_size_{}".format(mod)], mod)
+                                            inf_params["CNV_probabilities"],
+                                            cat_vector,
+                                            inf_params["NB_size_{}".format(mod)], mod)
+            else:
+                lk = lks.NB_likelihood_aux(self, segment_fact_marg, inf_params["NB_size_{}".format(mod)], mod)
+
+
 
         elif self._params["likelihood_{}".format(mod)] == "P":
-            if self._params["latent_type"] == "D":
+            if self._params["latent_type"] == "M":
                 lk = lks.poisson_likelihood_aux2(self, inf_params["segment_factor_{}".format(mod)],
                                             inf_params["CNV_probabilities"],
                                             cat_vector, mod)
@@ -316,7 +388,7 @@ class LatentCategorical(Model):
                 lk = lks.poisson_likelihood_aux(self, segment_fact_marg, mod)
 
         else:
-            if self._params["latent_type"] == "D":
+            if self._params["latent_type"] == "M":
                 lk = lks.gaussian_likelihood_aux2(self, inf_params["segment_factor_{}".format(mod)],
                                                  inf_params["CNV_probabilities"],
                                                  cat_vector,
