@@ -128,8 +128,12 @@ def log_sum_exp(args):
     return c + torch.log(torch.sum(torch.exp(args - c), axis=0))
 
 
-def entropy_CNV(x):
+def entropy_mixture(x):
 
     entr = torch.sum(x + torch.log(x + 1e-10), dim = 0)
-    entr = torch.mean(entr, dim = 1)
-    return entr.sum()
\ No newline at end of file
+    return entr.sum()
+
+
+def entropy_per_segment(x):
+    entr = torch.sum(x + torch.log(x + 1e-10), dim=0)
+    return entr.sum()
